<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2005 Larry Adams                                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/

/* get_catalyst_doet1dTpFdbEntry_ports
	obtains port associations for Cisco Catalyst Swtiches.  Catalyst
	switches are unique in that they support a different snmp_readstring for
	every VLAN interface on the switch.
*/
function get_catalyst_dot1dTpFdbEntry_ports($site, &$device, $lowPort = 0, $highPort = 0) {
	global $debug, $scan_date;

	/* initialize port counters */
	$device["ports_total"] = 0;
	$device["ports_active"] = 0;
	$device["ports_trunk"] = 0;

	/* get VLAN status */
	$vlan_ids = mactrack_walk($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.4.1.9.9.46.1.3.1.1.2", $device["snmp_version"],
					$device["snmp_timeout"], $device["snmp_retries"]);

	/* get VLAN names */
	$vlan_names = mactrack_walk($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.4.1.9.9.46.1.3.1.1.4", $device["snmp_version"],
					$device["snmp_timeout"], $device["snmp_retries"]);

	/* get VLAN Trunk status */
	$vlan_trunkstatus = xform_standard_indexed_data(".1.3.6.1.4.1.9.9.46.1.6.1.1.14", $device);
	$device["vlans_total"] = sizeof($vlan_trunkstatus);
	debug("VLAN data collected. There are " . (sizeof($vlan_ids)-3) . " VLANS.");

	/* get the ifIndexes for the device */
	$ifIndexes = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.1", $device);
	debug("ifIndexes data collection complete");

	/* get the ifTypes for the device */
	$ifTypes = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.3", $device);
	debug("ifTypes data collection complete.");

	/* get the ifNames for the device */
	$ifNames = xform_standard_indexed_data(".1.3.6.1.2.1.31.1.1.1.1", $device);
	debug("ifNames data collection complete.");

	foreach($ifIndexes as $ifIndex) {
		$ifInterfaces[$ifIndex]["ifIndex"] = $ifIndex;
		$ifInterfaces[$ifIndex]["ifName"] = @$ifNames[$ifIndex];
		$ifInterfaces[$ifIndex]["ifType"] = $ifTypes[$ifIndex];
		$ifInterfaces[$ifIndex]["trunkPortState"] = @$vlan_trunkstatus[$ifIndex];
	}
	debug("ifInterfaces assembly complete.");

	/* calculate the number of end user ports */
	foreach ($ifTypes as $ifType) {
		if ($ifType == 6) {
			$device["ports_total"]++;
		}
	}

	/* get the portNames */
	$portNames = xform_cisco_workgroup_port_data(".1.3.6.1.4.1.9.5.1.4.1.1.4", $device);

	debug("portNames data collected.");

	/* get trunking status */
	$portTrunking = xform_cisco_workgroup_port_data(".1.3.6.1.4.1.9.5.1.9.3.1.8", $device);

	debug("portTrunking data collected.");

	/* calculate the number of end user ports */
	foreach ($portTrunking as $portTrunk) {
		if ($portTrunk == 1) {
			$device["ports_trunk"]++;
		}
	}

	/* build VLAN array from results */
	$i = 0;
	$j = 0;
	$active_vlans = array();
	foreach($vlan_ids as $vlan_entry) {
		$vlanIDOID = $vlan_entry["oid"];
		$vlanStatus = $vlan_entry["value"];
        $vlanNameOID = $vlan_names[$i]["oid"];
		$vlanNameName = $vlan_names[$i]["value"];

		if ($vlanStatus == 1) { /* vlan is operatinal */
			$perPos = strrpos($vlanIDOID, ".");
			$vlan_number = substr($vlanIDOID, $perPos+1);
			switch ($vlan_number) {
			case "1002":
			case "1003":
			case "1004":
			case "1005":
				$active_vlan_ports = 0;
				break;
			default:
				$snmp_readstring = $device["snmp_readstring"] . "@" . $vlan_number;
				$active_vlan_ports = cacti_snmp_get($device["hostname"], $snmp_readstring,
										".1.3.6.1.2.1.17.1.2.0", $device["snmp_version"],
										"", "", 161, $device["snmp_timeout"]);

				if (!is_numeric($active_vlan_ports)) {
					$active_vlan_ports = 0;
				}else{
					$active_vlan_ports = $active_vlan_ports - $device["ports_trunk"];
				}

				if ($active_vlan_ports < 0) {
					$active_vlan_ports = 0;
				}

				debug("VLAN Analysis for VLAN: " . $vlan_number . " is complete. ACTIVE PORTS: " . $active_vlan_ports);

				if ($active_vlan_ports > 0) { /* does the vlan have active ports on it */
					$active_vlans[$j]["vlan_id"] = $vlan_number;
					$active_vlans[$j]["vlan_name"] = $vlanNameName;
					$active_vlans[$j]["active_ports"] = $active_vlan_ports;
					$active_vlans++;

					$j++;
				}
			}
		}

		$i++;
	}

	if (sizeof($active_vlans) > 0) {
		$i = 0;
		/* get the port status information */
		foreach($active_vlans as $active_vlan) {
			$snmp_readstring = $device["snmp_readstring"] . "@" . $active_vlan["vlan_id"];

			debug("Processing has begun for VLAN: " . $active_vlan["vlan_id"]);

			if ($highPort == 0) {
				$active_vlans[$i]["port_results"] = get_base_dot1dTpFdbEntry_ports($site, $device, $ifInterfaces, $snmp_readstring, FALSE);
			}else {
				$active_vlans[$i]["port_results"] = get_base_dot1dTpFdbEntry_ports($site, $device, $ifInterfaces, $snmp_readstring, FALSE, $lowPort, $highPort);
			}

			/* get bridge port mappings */
			/* get bridge port to ifIndex mappings */
			debug("Bridge port information about to be collected.");
			debug("VLAN_ID: " . $active_vlans[$i]["vlan_id"] . ", VLAN_NAME: " . $active_vlans[$i]["vlan_name"] . ", ACTIVE PORTS: " . sizeof($active_vlans[$i]["port_results"]));
			$brPortOID = ".1.3.6.1.2.1.17.1.4.1.2";
			$brPorttoifIndexes[$i] = mactrack_walk($device["hostname"], $snmp_readstring,
						$brPortOID, $device["snmp_version"], $device["snmp_timeout"], $device["snmp_retries"]);

			$j = 0;
			foreach($brPorttoifIndexes[$i] as $brPortEntry) {
				$perPos = strrpos($brPortEntry["oid"], ".");
				$brPortID = substr($brPortEntry["oid"], $perPos+1);
				$brPorttoifIndexes[$i][$j]["oid"] = $brPortID;
				$j++;
			}
			$brPorttoifIndexes[$i] = array_rekey($brPorttoifIndexes[$i], "oid", "value");
			debug("Bridge port information collection complete.");

			$i++;
		}

		$i = 0;
		$j = 0;
		$port_array = array();
		foreach($active_vlans as $active_vlan) {
			foreach($active_vlan["port_results"] as $port_result) {
				$ifIndex = $brPorttoifIndexes[$j][$port_result["port_number"]];
				$ifType = $ifTypes[$ifIndex];
				$ifName = $ifNames[$ifIndex];
				$portName = @$portNames[$ifName];
				$portTrunk = @$portTrunking[$ifName];
				$portTrunkStatus = @$ifInterfaces[$ifIndex]["trunkPortState"];

				/* only output legitamate end user ports */
				if ((($ifType == 6) && ($portTrunk == 2) && (substr_count($device["snmp_sysDescr"], "Catalyst") > 0)) ||
					(($ifType == 6) && (substr_count($device["snmp_sysDescr"], "IOS")))) {
					if (($portTrunkStatus == "2")||($portTrunkStatus == "4")||($portTrunkStatus =="")) {
						$port_array[$i]["vlan_id"] = $active_vlan["vlan_id"];
						$port_array[$i]["vlan_name"] = $active_vlan["vlan_name"];
						$port_array[$i]["port_number"] = $ifNames[$ifIndex];
						$port_array[$i]["port_name"] = $portName;
						$port_array[$i]["mac_address"] = $port_result["mac_address"];
						$device["ports_active"]++;
						$i++;

						debug("VLAN: " . $active_vlan["vlan_id"] . ", " .
							"NAME: " . $active_vlan["vlan_name"] . ", " .
							"PORT: " . $ifNames[$ifIndex] . ", " .
							"NAME: " . $portName . ", " .
							"MAC: " . $port_result["mac_address"]);
					}
				}
			}
			$j++;
		}

		/* display completion message */
		print("INFO: HOST: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", TOTAL PORTS: " . $device["ports_total"] . ", ACTIVE PORTS: " . $device["ports_active"]);
		$device["last_runmessage"] = "Data collection completed ok";

		db_store_device_port_results($device, $port_array, $scan_date);
	}else{
		print("INFO: HOST: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", No active devcies on this network device.");
		$device["snmp_status"] = HOST_UP;
		$device["last_runmessage"] = "Data collection completed ok. No active devices on this network device.";
	}

}

/* get_IOS_doet1dTpFdbEntry_ports
	obtains port associations for Cisco Catalyst Swtiches.  Catalyst
	switches are unique in that they support a different snmp_readstring for
	every VLAN interface on the switch.
*/
function get_IOS_dot1dTpFdbEntry_ports($site, &$device, $lowPort = 0, $highPort = 0) {
	global $debug, $scan_date;

	/* initialize port counters */
	$device["ports_total"] = 0;
	$device["ports_active"] = 0;
	$device["ports_trunk"] = 0;

	/* get VLAN status */
	$vlan_ids = mactrack_walk($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.4.1.9.9.46.1.3.1.1.2", $device["snmp_version"],
					$device["snmp_timeout"], $device["snmp_retries"]);

	/* get VLAN names */
	$vlan_names = mactrack_walk($device["hostname"], $device["snmp_readstring"],
					".1.3.6.1.4.1.9.9.46.1.3.1.1.4", $device["snmp_version"],
					$device["snmp_timeout"], $device["snmp_retries"]);

	/* get VLAN Trunk status */
	$vlan_trunkstatus = xform_standard_indexed_data(".1.3.6.1.4.1.9.9.46.1.6.1.1.13", $device);
	$device["vlans_total"] = sizeof($vlan_trunkstatus);
	debug("VLAN data collected. There are " . (sizeof($vlan_ids)-3) . " VLANS.");

	/* get the ifIndexes for the device */
	$ifIndexes = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.1", $device);
	debug("ifIndex data collection complete");

	/* get the ifTypes for the device */
	$ifTypes = xform_standard_indexed_data(".1.3.6.1.2.1.2.2.1.3", $device);
	debug("ifTypes data collection complete.");

	/* get the ifNames for the device */
	$ifNames = xform_standard_indexed_data(".1.3.6.1.2.1.31.1.1.1.1", $device);
	debug("ifNames data collection complete.");

	/* get ports names through use of ifAlias */
	$ifAlias = xform_standard_indexed_data(".1.3.6.1.2.1.31.1.1.1.18", $device);
	debug("ifAlias data collection complete.");

	foreach($ifIndexes as $ifIndex) {
		$ifInterfaces[$ifIndex]["ifIndex"] = $ifIndex;
		$ifInterfaces[$ifIndex]["ifName"] = @$ifNames[$ifIndex];
		$ifInterfaces[$ifIndex]["ifType"] = $ifTypes[$ifIndex];
		$ifInterfaces[$ifIndex]["ifAlias"] = $ifAlias[$ifIndex];
		$ifInterfaces[$ifIndex]["trunkPortState"] = @$vlan_trunkstatus[$ifIndex];
	}
	debug("ifInterfaces assembly complete.");

	/* calculate the number of end user ports */
	foreach ($ifTypes as $ifType) {
		if ($ifType == 6) {
			$device["ports_total"]++;
		}
	}

	/* calculate the number of end vlan ports */
	foreach ($vlan_trunkstatus as $portTrunk) {
		if ($portTrunk == "1") {
			$device["ports_trunk"]++;
		}
	}

	/* build VLAN array from results */
	$i = 0;
	$j = 0;
	$active_vlans = array();
	foreach($vlan_ids as $vlan_entry) {
		$vlanIDOID = $vlan_entry["oid"];
		$vlanStatus = $vlan_entry["value"];
		$vlanNameOID = $vlan_names[$i]["oid"];
		$vlanNameName = $vlan_names[$i]["value"];

		if ($vlanStatus == 1) { /* vlan is operatinal */
			$perPos = strrpos($vlanIDOID, ".");
			$vlan_number = substr($vlanIDOID, $perPos+1);
			switch ($vlan_number) {
			case "1002":
			case "1003":
			case "1004":
			case "1005":
				$active_vlan_ports = 0;
				break;
			default:
				$snmp_readstring = $device["snmp_readstring"] . "@" . $vlan_number;
				$active_vlan_ports = cacti_snmp_get($device["hostname"], $snmp_readstring,
										".1.3.6.1.2.1.17.1.2.0", $device["snmp_version"],
										"", "", 161, $device["snmp_timeout"]);

				if (!is_numeric($active_vlan_ports)) {
					$active_vlan_ports = 0;
				}else{
					$active_vlan_ports = $active_vlan_ports - $device["ports_trunk"];
				}

				if ($active_vlan_ports < 0) {
					$active_vlan_ports = 0;
				}

				debug("VLAN Analysis for VLAN: " . $vlan_number . " is complete. ACTIVE PORTS: " . $active_vlan_ports);

				if ($active_vlan_ports > 0) { /* does the vlan have active ports on it */
					$active_vlans[$j]["vlan_id"] = $vlan_number;
					$active_vlans[$j]["vlan_name"] = $vlanNameName;
					$active_vlans[$j]["active_ports"] = $active_vlan_ports;
					$active_vlans++;

					$j++;
				}
			}
		}

		$i++;
	}

	if (sizeof($active_vlans) > 0) {
		$i = 0;
		/* get the port status information */
		foreach($active_vlans as $active_vlan) {
			$snmp_readstring = $device["snmp_readstring"] . "@" . $active_vlan["vlan_id"];

			debug("Processing has begun for VLAN: " . $active_vlan["vlan_id"]);
			if ($highPort == 0) {
				$active_vlans[$i]["port_results"] = get_base_dot1dTpFdbEntry_ports($site, $device, $ifInterfaces, $snmp_readstring, FALSE);
			}else {
				$active_vlans[$i]["port_results"] = get_base_dot1dTpFdbEntry_ports($site, $device, $ifInterfaces, $snmp_readstring, FALSE, $lowPort, $highPort);
			}

			/* get bridge port mappings */
			/* get bridge port to ifIndex mappings */
			debug("Bridge port information about to be collected.");
			debug("VLAN_ID: " . $active_vlans[$i]["vlan_id"] . ", VLAN_NAME: " . $active_vlans[$i]["vlan_name"] . ", ACTIVE PORTS: " . sizeof($active_vlans[$i]["port_results"]));

			if (sizeof($active_vlans[$i]["port_results"]) > 0) {
				$brPortOID = ".1.3.6.1.2.1.17.1.4.1.2";
				$brPorttoifIndexes[$i] = mactrack_walk($device["hostname"], $snmp_readstring,
							$brPortOID, $device["snmp_version"], $device["snmp_timeout"], $device["snmp_retries"]);

				$j = 0;
				foreach($brPorttoifIndexes[$i] as $brPortEntry) {
					$perPos = strrpos($brPortEntry["oid"], ".");
					$brPortID = substr($brPortEntry["oid"], $perPos+1);
					$brPorttoifIndexes[$i][$j]["oid"] = $brPortID;
					$j++;
				}
				$brPorttoifIndexes[$i] = array_rekey($brPorttoifIndexes[$i], "oid", "value");
				debug("Bridge port information collection complete.");
			}
			$i++;
		}

		$i = 0;
		$j = 0;
		$port_array = array();

		foreach($active_vlans as $active_vlan) {
			if (sizeof($active_vlans[$j]["port_results"]) > 0) {
				foreach($active_vlan["port_results"] as $port_result) {
					$ifIndex = $brPorttoifIndexes[$j][$port_result["port_number"]];
					$ifType = $ifTypes[$ifIndex];
					$portNumber = $ifNames[$ifIndex];
					$portName = @$ifAlias[$ifIndex];
					$portTrunk = @$portTrunking[$ifName];
					$portTrunkStatus = @$ifInterfaces[$ifIndex]["trunkPortState"];

					/* only output legitamate end user ports */
					if ((($ifType == 6) && ($portTrunk == 2) && (substr_count($device["snmp_sysDescr"], "Catalyst") > 0)) ||
						(($ifType == 6) && (substr_count($device["snmp_sysDescr"], "IOS")))) {
						/* don't count ports with IP's associated with them */
						if ($portTrunkStatus == "2") {
							$port_array[$i]["vlan_id"] = $active_vlan["vlan_id"];
							$port_array[$i]["vlan_name"] = $active_vlan["vlan_name"];
							$port_array[$i]["port_number"] = $portNumber;
							$port_array[$i]["port_name"] = $portName;
							$port_array[$i]["mac_address"] = $port_result["mac_address"];
							$device["ports_active"]++;
							$i++;

							debug("VLAN: " . $active_vlan["vlan_id"] . ", " .
								"NAME: " . $active_vlan["vlan_name"] . ", " .
								"PORT: " . $portNumber . ", " .
								"NAME: " . $portName . ", " .
								"MAC: " . $port_result["mac_address"]);
						}
					}
				}

			} else {
				debug("WARNING Active VLAN Error.");
			}
			$j++;
		}

		/* display completion message */
		print("INFO: HOST: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", TOTAL PORTS: " . $device["ports_total"] . ", ACTIVE PORTS: " . $device["ports_active"]);
		$device["last_runmessage"] = "Data collection completed ok";

		db_store_device_port_results($device, $port_array, $scan_date);
	}else{
		print("INFO: HOST: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", No active end devices on this device.");
		$device["snmp_status"] = HOST_UP;
		$device["last_runmessage"] = "Data collection completed ok.  No active end devices on this device.";
	}
}

?>