<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2005 Larry Adams                                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}
//chdir('../../');
$no_http_headers = true;

include("./include/config.php");
include_once($config["base_path"] . "/lib/snmp.php");
include_once($config["base_path"] . "/lib/ping.php");
include_once($config["base_path"] . "/lib/functions.php");

include_once($config["base_path"] . "/plugins/mac_track/lib/mactrack_functions.php");
include_once($config["base_path"] . "/plugins/mac_track/lib/mactrack_cabletron.php");
include_once($config["base_path"] . "/plugins/mac_track/lib/mactrack_cisco.php");
if (!defined("HOST_ERROR"))
	define("HOST_ERROR", 4);
/* Let the scanner run for no more that 25 minutes */
ini_set("max_execution_time", 1500);

/* obtain some date/times for later use */
$scan_date = read_config_option("mactrack_scan_date");
list($micro,$seconds) = split(" ", microtime());
$start_time = $seconds + $micro;

/* drop a few environment variables to minimize net-snmp load times */
putenv("MIBS=RFC-1215");
ini_set("max_execution_time", "0");
ini_set("memory_limit", "32M");

/* establish constants */
define("DEVICE_HUB_SWITCH", 1);
define("DEVICE_SWITCH_ROUTER", 2);
define("DEVICE_ROUTER", 3);

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

/* utility requires input parameters */
if (sizeof($parms) == 0) {
	print "ERROR: You must supply input parameters\n\n";
	display_help();
	exit;
}

$debug = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);

	switch ($arg) {
	case "-id":
		$device_id = $value;
		break;
	case "-d":
		$debug = TRUE;
		break;
	case "-h":
		display_help();
		exit;
	case "-v":
		display_help();
		exit;
	case "--version":
		display_help();
		exit;
	case "--help":
		display_help();
		exit;
	default:
		print "ERROR: Invalid Parameter " . $parameter . "\n\n";
		display_help();
		exit;
	}
}

/* place a process marker in the database */
db_process_add($device_id);

/* get device information */
$device = db_fetch_row("SELECT * FROM mac_track_devices WHERE device_id =" . $device_id);
if (sizeof($device) == 0) {
	print "ERROR: Device with Id of '$device_id' not found in database.  Can not continue.\n";
	db_process_remove($device_id);
	exit;
}

/* get the site name */
$site = db_fetch_cell("SELECT site_name FROM mac_track_sites WHERE site_id ='" . $device["site_id"] . "'");
if (strlen($site) == 0) {
	print "ERROR: Site not found in database. Can not continue.\n";
	cacti_log("ERROR: Site not found in database. Can not continue.\n",true, "POLLER");
	db_process_remove($device_id);
	exit;
}

/* get device types */
$device_types = db_fetch_cell("SELECT * FROM mac_track_device_types");
if (strlen($device_types) == 0) {
	print "ERROR: No device types have been found.\n";
	cacti_log("ERROR: No device types have been found.\n",true, "POLLER");
	db_process_remove($device_id);
	exit;
}

/* check the devices read string for validity, set to new if changed */
if (valid_snmp_device($device)) {
	debug("HOST: " . $device["hostname"] . " is alive, processing has begun.");
	$host_up = TRUE;

	if (($device["scan_type"] == DEVICE_HUB_SWITCH) || ($device["scan_type"] == DEVICE_SWITCH_ROUTER)) {
		if (substr_count($device["snmp_sysDescr"], "SEHI")) {
			get_repeater_rev4_ports($site, $device, 1, 24);
		}elseif (substr_count($device["snmp_sysDescr"], "6H252-17")) {
			get_cabletron_switch_ports($site, $device, 1, 16);
		}elseif ((substr_count($device["snmp_sysDescr"], "2E253-49R")) ||
				(substr_count($device["snmp_sysDescr"], "6E233-49"))) {
			get_cabletron_switch_ports($site, $device, 1, 48);
		}elseif ((substr_count($device["snmp_sysDescr"], "6E132-25")) ||
				(substr_count($device["snmp_sysDescr"], "2H252-25R")) ||
				(substr_count($device["snmp_sysDescr"], "6E138-25")) ||
				(substr_count($device["snmp_sysDescr"], "6H352-25")) ||
				(substr_count($device["snmp_sysDescr"], "2E42-27R")) ||
				(substr_count($device["snmp_sysDescr"], "6H202-24"))) {
			get_cabletron_switch_ports($site, $device, 1, 24);
		}elseif (substr_count($device["snmp_sysDescr"], "6H122-08")) {
			get_cabletron_switch_ports($site, $device, 1, 8);
		}elseif (substr_count($device["snmp_sysDescr"], "VH-2402S")) {
			get_generic_switch_ports($site, $device, 2, 24);
		}elseif (substr_count($device["snmp_sysDescr"], "IOS")) {
			if ((substr_count($device["snmp_sysDescr"], "C2950")) ||
				(substr_count($device["snmp_sysDescr"], "C3750")) ||
				(substr_count($device["snmp_sysDescr"], "C3550")) ||
				(substr_count($device["snmp_sysDescr"], "C3560"))) {
				get_IOS_dot1dTpFdbEntry_ports($site, $device);
			}elseif (substr_count($device["snmp_sysDescr"], "C2900XL")) {
				get_IOS_dot1dTpFdbEntry_ports($site, $device, 1, 24);
			}elseif (substr_count($device["snmp_sysDescr"], "c6sup2")) {
				get_IOS_dot1dTpFdbEntry_ports($site, $device);
			}else{
				print("ERROR: SITE: " . $site . ", IP: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", Unsupported device.\n");
				cacti_log("ERROR: SITE: " . $site . ", IP: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", Unsupported device.\n",true, "POLLER");
				$device["last_runmessage"] = "ERROR: Unsupported device.";
				$device["snmp_status"] = HOST_ERROR;
			}
		}elseif (substr_count($device["snmp_sysDescr"], "Catalyst")) {
			if ((substr_count($device["snmp_sysDescr"], "1900")) ||
				(substr_count($device["snmp_sysDescr"], "2820"))) {
				get_generic_switch_ports($site, $device, 1, 24);
			}else{
				get_catalyst_dot1dTpFdbEntry_ports($site, $device);
			}
		}else{
			print("ERROR: SITE: " . $site . ", IP: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", Unsupported device.\n");
			$device["last_runmessage"] = "ERROR: Unsupported device.";
			$device["snmp_status"] = HOST_ERROR;
		}
	}
	/* push ARP table to mac_track_ip table */
	if (($device["scan_type"] == DEVICE_SWITCH_ROUTER) ||
		($device["scan_type"] == DEVICE_ROUTER)) {

		get_arp_ip_ranges($site, $device);
	}
}else{
	print("ERROR: SITE: " . $site . ", IP: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", Device unreachable.\n");
	cacti_log("ERROR: SITE: " . $site . ", IP: " . $device["hostname"] . ", TYPE: " . substr($device["snmp_sysDescr"],0,40) . ", Device unreachable.",true, "POLLER");
	$host_up = FALSE;
}

/* update the database with device status information */
db_update_device_status($device, $host_up, $scan_date, $start_time);
db_process_remove($device_id);
exit;

/*	display_help - displays the usage of the function */
function display_help () {
	print "Network Mac Tracker Version 1.0, Copyright 2005 - Larry Adams\n\n";
	print "usage: mactrack_device.php -id=host_id [-d] [-h] [--help] [-v] [--version]\n\n";
	print "-id=host_id   - the mac_track_devices host_id to scan\n";
	print "-d            - Display verbose output during execution\n";
	print "-v --version  - Display this help message\n";
	print "-h --help     - display this help message\n";
}

function debug($message) {
	global $debug;

	if ($debug) {
		print("DEBUG: " . $message . "\n");
	}
}

?>